/* OscThread -- thread for making an oscillator go go go
 *	this one has a filter!
 *
 * 		Brad Garton, spring 2003
 *
*/
import com.softsynth.jsyn.*;

class OscThread extends Thread
{
	SawtoothOscillator	theOsc;
	SynthEnvelope		theEnv;
	EnvelopePlayer		theEnvPlayer;
	double [] envdata =
	{
		0.0, 0.0,
		0.5, 1.0,
		1.0, 1.0,
		0.5, 0.0
	};
	MultiplyUnit		theMult;
	LineOut			oscOut;

	public OscThread() throws SynthException
	{
		theOsc = new SawtoothOscillator();

		// envdata has already been set up
		theEnv = new SynthEnvelope(envdata);
		theEnvPlayer = new EnvelopePlayer();

		theMult = new MultiplyUnit();
		oscOut = new LineOut();

		// connect 'em up!
		theOsc.output.connect(theMult.inputA);
		theEnvPlayer.output.connect(theMult.inputB);

		theMult.output.connect(0, oscOut.input, 0);
		theMult.output.connect(0, oscOut.input, 1);
	}

	public void run()
	{
		try
		{
			theOsc.start();
			theEnvPlayer.start();
			theMult.start();
			oscOut.start();
		} catch(SynthException e) {
			SynthAlert.showError(e);
		}
	}

	public void go(int start, double dur, double freq, double amp, double rise, double decay)
	{
		// we will write new values into this array to reflect the
		// "rise", "decay" and "dur" params passed in from above
		double [] envdata2 =
		{
			0.0, 0.0,
			0.5, 1.0,
			1.0, 1.0,
			0.5, 0.0
		};

		// start in ticks, dur/rise/decay in seconds, freq in Hz
		try
		{
			// we use "start" to set up the point in time when
			// these parameters will actually be changed
			theOsc.frequency.set(start, freq);
			theOsc.amplitude.set(start, amp);

			// and here is where we rewrite the amplitude envelope
			// based on the parameters we get
			envdata2[2] = rise;
			envdata2[4] = dur-(rise+decay);
			envdata2[6] = decay;
			// unfortunately writing the data into the
			// SynthEnvelope can't be timed... so we need to
			// be aware of this in the 'outer' program
			theEnv.write(envdata2);

			theEnvPlayer.envelopePort.clear(start);
			theEnvPlayer.envelopePort.queue(start, theEnv);
		} catch(SynthException e) {
			SynthAlert.showError(e);
		}
	}
}
